/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.registry.ActionConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Handles user related operations in the registry. Checks if the actor is authorized to perform
 * the requested operation and delegated it to the realm, if authorization succeeds.
 */
public class RegistryUserManager {

    private static final Log log = LogFactory.getLog(RegistryUserManager.class);

    private Realm realm;
    private String actor;

    public RegistryUserManager(Realm realm, String userName) {
        this.realm = realm;
        this.actor = userName;
    }

    public void addUser(String userName, String password) throws RegistryException {

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.ADD_USER)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.addUser(userName, password);
        } catch (UserManagerException e) {
            String msg = "Could not add the user. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeUser(String userName) throws RegistryException {

        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot remove the administrator.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_USER)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.deleteUser(userName);            
        } catch (UserManagerException e) {
            String msg = "Could not delete the user. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public String[] getAllUsers() throws RegistryException {

        try {
            String[] userNames = realm.getAllUserNames();
            return userNames;

        } catch (UserManagerException e) {
            String msg = "Could not get all users of the Registry. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void authorizeUser(String userName, String resourceID, String action)
            throws RegistryException {

        try {
            if (!realm.isUserAuthorized(actor, resourceID, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.authorizeUser(userName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not authorize. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void denyUser(String userName, String resourceID, String action)
            throws RegistryException {

        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot deny access for the administrator.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            if (!realm.isUserAuthorized(actor, resourceID, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.denyUser(userName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not deny. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }
}
